from Namespace import Namespace

ShpDefinitions = Namespace(
  TagOpen = '[',
  TagClose = ']',
  TagNameScoped = '$',
  TagNameScopeless = '%',
  TagId = '#',
  TagClass = '.',
  TagFlagParam = '!',
  ScopeOpen = '{',
  ScopeClose = '}',
  FunctionName = '@',
  Literal = '\'',
  Comment = '//',
  DefaultDoctype = 'HTML',
)
