from setuptools import setup, find_packages

setup(
  name = 'shp',
  description = 'Static-Html-Preprocessor',
  version = '1.0.2',
  license = 'MIT',
  author = 'Jakub21',
  url = 'https://github.com/Jakub21/shp.py',
  packages = find_packages(),
)
